@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,9 @@
 #include <strings.h>
 #include <stdlib.h>
 #include <sys/types.h>
-#if HAVE_REGEX_H
+#if USE_ONIGURUMA
+#  include <onigposix.h>
+#elif HAVE_REGEX_H
 #  include <regex.h>
 #endif
 #include <time.h>
@@ -95,9 +97,13 @@ gint filter_apply_msginfo(GSList *fltlist, MsgInfo *msginfo,
 	g_return_val_if_fail(msginfo != NULL, -1);
 	g_return_val_if_fail(fltinfo != NULL, -1);
 
+	fltinfo->error = FLT_ERROR_OK;
+
 	if (!fltlist) return 0;
 
 	file = procmsg_get_message_file(msginfo);
+	if (!file)
+		return -1;
 	hlist = procheader_get_header_list_from_file(file);
 	if (!hlist) {
 		g_free(file);
@@ -105,12 +111,18 @@ gint filter_apply_msginfo(GSList *fltlist, MsgInfo *msginfo,
 	}
 
 	for (cur = fltlist; cur != NULL; cur = cur->next) {
+		gboolean matched;
+
 		rule = (FilterRule *)cur->data;
 		if (!rule->enabled) continue;
-		if (filter_match_rule(rule, msginfo, hlist, fltinfo)) {
+		matched = filter_match_rule(rule, msginfo, hlist, fltinfo);
+		if (fltinfo->error != FLT_ERROR_OK) {
+			g_warning("filter_match_rule() returned error (code: %d)\n", fltinfo->error);
+		}
+		if (matched) {
 			ret = filter_action_exec(rule, msginfo, file, fltinfo);
 			if (ret < 0) {
-				g_warning("filter_action_exec() returned error\n");
+				g_warning("filter_action_exec() returned error (code: %d)\n", fltinfo->error);
 				break;
 			}
 			if (fltinfo->drop_done == TRUE ||
@@ -198,6 +210,7 @@ gint filter_action_exec(FilterRule *rule, MsgInfo *msginfo, const gchar *file,
 			if (!dest_folder) {
 				g_warning("dest folder '%s' not found\n",
 					  action->str_value);
+				fltinfo->error = FLT_ERROR_ERROR;
 				return -1;
 			}
 			debug_print("filter_action_exec(): %s: dest_folder = %s\n",
@@ -212,18 +225,27 @@ gint filter_action_exec(FilterRule *rule, MsgInfo *msginfo, const gchar *file,
 					copy_to_self = TRUE;
 				else {
 					if (action->type == FLT_ACTION_COPY) {
+						MsgFlags save_flags;
+
+						save_flags = msginfo->flags;
+						msginfo->flags = fltinfo->flags;
 						val = folder_item_copy_msg
 							(dest_folder, msginfo);
-						if (val == -1)
+						msginfo->flags = save_flags;
+						if (val == -1) {
+							fltinfo->error = FLT_ERROR_ERROR;
 							return -1;
+						}
 					}
 					fltinfo->actions[action->type] = TRUE;
 				}
 			} else {
 				if (folder_item_add_msg(dest_folder, file,
 							&fltinfo->flags,
-							FALSE) < 0)
+							FALSE) < 0) {
+					fltinfo->error = FLT_ERROR_ERROR;
 					return -1;
+				}
 				fltinfo->actions[action->type] = TRUE;
 			}
 
@@ -276,20 +298,21 @@ gint filter_action_exec(FilterRule *rule, MsgInfo *msginfo, const gchar *file,
 
 static gboolean strmatch_regex(const gchar *haystack, const gchar *needle)
 {
-#if HAVE_REGEX_H && HAVE_REGCOMP
+#if defined(USE_ONIGURUMA) || defined(HAVE_REGCOMP)
 	gint ret = 0;
 	regex_t preg;
-	regmatch_t pmatch[1];
 
-	ret = regcomp(&preg, needle, REG_EXTENDED|REG_ICASE);
-	if (ret != 0) return FALSE;
+#if USE_ONIGURUMA
+	reg_set_encoding(REG_POSIX_ENCODING_UTF8);
+#endif
+	ret = regcomp(&preg, needle, REG_ICASE|REG_EXTENDED);
+	if (ret != 0)
+		return FALSE;
 
-	ret = regexec(&preg, haystack, 1, pmatch, 0);
+	ret = regexec(&preg, haystack, 0, NULL, 0);
 	regfree(&preg);
 
-	if (ret == REG_NOMATCH) return FALSE;
-
-	if (pmatch[0].rm_so != -1)
+	if (ret == 0)
 		return TRUE;
 	else
 #endif
@@ -348,6 +371,7 @@ gboolean filter_match_rule(FilterRule *rule, MsgInfo *msginfo, GSList *hlist,
 static gboolean filter_match_cond(FilterCond *cond, MsgInfo *msginfo,
 				  GSList *hlist, FilterInfo *fltinfo)
 {
+	gint ret;
 	gboolean matched = FALSE;
 	gchar *file;
 	gchar *cmdline;
@@ -364,8 +388,13 @@ static gboolean filter_match_cond(FilterCond *cond, MsgInfo *msginfo,
 		break;
 	case FLT_COND_CMD_TEST:
 		file = procmsg_get_message_file(msginfo);
+		if (!file)
+			return FALSE;
 		cmdline = g_strconcat(cond->str_value, " \"", file, "\"", NULL);
-		matched = (execute_command_line(cmdline, FALSE) == 0);
+		ret = execute_command_line(cmdline, FALSE);
+		matched = (ret == 0);
+		if (ret == -1)
+			fltinfo->error = FLT_ERROR_EXEC_FAILED;
 		g_free(cmdline);
 		g_free(file);
 		break;
@@ -395,6 +424,7 @@ static gboolean filter_match_cond(FilterCond *cond, MsgInfo *msginfo,
 	default:
 		g_warning("filter_match_cond(): unknown condition: %d\n",
 			  cond->type);
+		fltinfo->error = FLT_ERROR_ERROR;
 		return FALSE;
 	}
 
@@ -1247,6 +1277,7 @@ FilterInfo *filter_info_new(void)
 	fltinfo->dest_list = NULL;
 	fltinfo->move_dest = NULL;
 	fltinfo->drop_done = FALSE;
+	fltinfo->error = FLT_ERROR_OK;
 
 	return fltinfo;
 }
@@ -1272,15 +1303,14 @@ void filter_rule_rename_dest_path(FilterRule *rule, const gchar *old_path,
 		if (action->str_value &&
 		    !strncmp(old_path, action->str_value, oldpathlen)) {
 			base = action->str_value + oldpathlen;
-			if (*base != G_DIR_SEPARATOR && *base != '\0')
+			if (*base != '/' && *base != '\0')
 				continue;
-			while (*base == G_DIR_SEPARATOR) base++;
+			while (*base == '/') base++;
 			if (*base == '\0')
 				dest_path = g_strdup(new_path);
 			else
-				dest_path = g_strconcat(new_path,
-							G_DIR_SEPARATOR_S,
-							base, NULL);
+				dest_path = g_strconcat(new_path, "/", base,
+							NULL);
 			debug_print("filter_rule_rename_dest_path(): "
 				    "renaming %s -> %s\n",
 				    action->str_value, dest_path);
@@ -1309,7 +1339,7 @@ void filter_rule_delete_action_by_dest_path(FilterRule *rule, const gchar *path)
 
 		if (action->str_value &&
 		    !strncmp(path, action->str_value, pathlen) &&
-		    (action->str_value[pathlen] == G_DIR_SEPARATOR ||
+		    (action->str_value[pathlen] == '/' ||
 		     action->str_value[pathlen] == '\0')) {
 			debug_print("filter_rule_delete_action_by_dest_path(): "
 				    "deleting %s\n", action->str_value);
