@@ -265,6 +265,9 @@ static gboolean session_ping_cb(gpointer data)
 	Session *session = SESSION(data);
 	SockInfo *sock = session->sock;
 
+	if (!session_is_connected(session))
+		return FALSE;
+
 	if (session->io_tag > 0 && sock && sock->callback) {
 		GTimeVal tv_cur, tv_result;
 
@@ -468,12 +471,22 @@ gint session_send_data(Session *session, FILE *data_fp, guint size)
 	session->write_data_len = size;
 	g_get_current_time(&session->tv_prev);
 
+#ifdef G_OS_WIN32
+	sock_set_nonblocking_mode(session->sock, FALSE);
+#endif
+
 	ret = session_write_data_cb(session->sock, G_IO_OUT, session);
 
 	if (ret == TRUE)
+#ifdef G_OS_WIN32
+		session->io_tag = sock_add_watch_poll(session->sock, G_IO_OUT,
+						      session_write_data_cb,
+						      session);
+#else
 		session->io_tag = sock_add_watch(session->sock, G_IO_OUT,
 						 session_write_data_cb,
 						 session);
+#endif
 	else if (session->state == SESSION_ERROR)
 		return -1;
 
@@ -990,7 +1003,7 @@ static gint session_write_buf(Session *session)
 
 #define WRITE_DATA_BUFFSIZE	8192
 
-static gint session_write_data(Session *session)
+static gint session_write_data(Session *session, gint *nwritten)
 {
 	gchar buf[WRITE_DATA_BUFFSIZE];
 	gint write_len;
@@ -1018,10 +1031,13 @@ static gint session_write_data(Session *session)
 		default:
 			g_warning("sock_write: %s\n", g_strerror(errno));
 			session->state = SESSION_ERROR;
+			*nwritten = write_len;
 			return -1;
 		}
 	}
 
+	*nwritten = write_len;
+
 	/* incomplete write */
 	if (session->write_data_pos + write_len < session->write_data_len) {
 		session->write_data_pos += write_len;
@@ -1077,6 +1093,7 @@ static gboolean session_write_data_cb(SockInfo *source,
 {
 	Session *session = SESSION(data);
 	guint write_data_len;
+	gint write_len;
 	gint ret;
 
 	g_return_val_if_fail(condition == G_IO_OUT, FALSE);
@@ -1086,7 +1103,7 @@ static gboolean session_write_data_cb(SockInfo *source,
 
 	write_data_len = session->write_data_len;
 
-	ret = session_write_data(session);
+	ret = session_write_data(session, &write_len);
 
 	if (ret < 0) {
 		session->state = SESSION_ERROR;
@@ -1121,5 +1138,9 @@ static gboolean session_write_data_cb(SockInfo *source,
 		session->send_data_notify(session, write_data_len,
 					  session->send_data_notify_data);
 
+#ifdef G_OS_WIN32
+	sock_set_nonblocking_mode(session->sock, session->nonblocking);
+#endif
+
 	return FALSE;
 }
