@@ -568,6 +568,23 @@ guint sock_add_watch(SockInfo *sock, GIOCondition condition, SockFunc func,
 	return g_io_add_watch(sock->sock_ch, condition, sock_watch_cb, sock);
 }
 
+guint sock_add_watch_poll(SockInfo *sock, GIOCondition condition, SockFunc func,
+			  gpointer data)
+{
+	GSource *source;
+
+	sock->callback = func;
+	sock->condition = condition;
+	sock->data = data;
+
+	source = g_source_new(&sock_watch_funcs, sizeof(SockSource));
+	((SockSource *)source)->sock = sock;
+	g_source_set_priority(source, G_PRIORITY_DEFAULT);
+	g_source_set_can_recurse(source, FALSE);
+
+	return g_source_attach(source, NULL);
+}
+
 static gint fd_check_io(gint fd, GIOCondition cond)
 {
 	struct timeval timeout;
@@ -1346,6 +1363,8 @@ gint ssl_read(SSL *ssl, gchar *buf, gint len)
 {
 	gint err, ret;
 
+	errno = 0;
+
 	if (SSL_pending(ssl) == 0) {
 		if (fd_check_io(SSL_get_rfd(ssl), G_IO_IN) < 0)
 			return -1;
@@ -1366,6 +1385,9 @@ gint ssl_read(SSL *ssl, gchar *buf, gint len)
 		g_warning("SSL_read() returned error %d, ret = %d\n", err, ret);
 		if (ret == 0)
 			return 0;
+#ifdef G_OS_WIN32
+		errno = EIO;
+#endif
 		return -1;
 	}
 }
