@@ -28,27 +28,30 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkvbox.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtkbutton.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtkhseparator.h>
+#include <gtk/gtkstock.h>
 
-#include "intl.h"
 #include "gtkutils.h"
+#include "stock_pixmap.h"
 #include "prefs_common.h"
 #include "addressadd.h"
 #include "addritem.h"
 #include "addrbook.h"
 #include "addrindex.h"
+#include "editaddress.h"
 #include "manage_window.h"
 
-#include "pixmaps/book.xpm"
-#include "pixmaps/dir-open.xpm"
-
 typedef struct {
 	AddressBookFile	*book;
 	ItemFolder	*folder;
@@ -62,8 +65,6 @@ static struct _AddressAdd_dlg {
 	GtkWidget *tree_folder;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
-	GtkWidget *statusbar;
-	gint status_cid;
 	FolderInfo *fiSelected;
 } addressadd_dlg;
 
@@ -88,29 +89,18 @@ static void addressadd_free_folderinfo( FolderInfo *fi ) {
 	g_free( fi );
 }
 
-/*
-* Edit functions.
-*/
-static void addressadd_status_show( gchar *msg ) {
-	if( addressadd_dlg.statusbar != NULL ) {
-		gtk_statusbar_pop( GTK_STATUSBAR(addressadd_dlg.statusbar), addressadd_dlg.status_cid );
-		if( msg ) {
-			gtk_statusbar_push( GTK_STATUSBAR(addressadd_dlg.statusbar), addressadd_dlg.status_cid, msg );
-		}
-	}
-}
-
 static gint addressadd_delete_event( GtkWidget *widget, GdkEventAny *event, gboolean *cancelled ) {
 	addressadd_cancelled = TRUE;
 	gtk_main_quit();
 	return TRUE;
 }
 
-static void addressadd_key_pressed( GtkWidget *widget, GdkEventKey *event, gboolean *cancelled ) {
+static gboolean addressadd_key_pressed( GtkWidget *widget, GdkEventKey *event, gboolean *cancelled ) {
 	if (event && event->keyval == GDK_Escape) {
 		addressadd_cancelled = TRUE;
 		gtk_main_quit();
 	}
+	return FALSE;
 }
 
 static void addressadd_ok( GtkWidget *widget, gboolean *cancelled ) {
@@ -129,8 +119,8 @@ static void addressadd_folder_select( GtkCTree *ctree, gint row, gint column,
 	addressadd_dlg.fiSelected = gtk_clist_get_row_data( GTK_CLIST(ctree), row );
 }
 
-static void addressadd_tree_button( GtkCTree *ctree, GdkEventButton *event, gpointer data ) {
-	if( ! event ) return;
+static gboolean addressadd_tree_button( GtkCTree *ctree, GdkEventButton *event, gpointer data ) {
+	if( ! event ) return FALSE;
 	if( event->button == 1 ) {
 		/* Handle double click */
 		if( event->type == GDK_2BUTTON_PRESS ) {
@@ -138,6 +128,8 @@ static void addressadd_tree_button( GtkCTree *ctree, GdkEventButton *event, gpoi
 			gtk_main_quit();
 		}
 	}
+
+	return FALSE;
 }
 
 static void addressadd_create( void ) {
@@ -155,30 +147,30 @@ static void addressadd_create( void ) {
 	GtkWidget *hsep;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
-	GtkWidget *hsbox;
-	GtkWidget *statusbar;
 	gint top;
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_widget_set_usize( window, 300, 400 );
-	gtk_container_set_border_width( GTK_CONTAINER(window), 0 );
-	gtk_window_set_title( GTK_WINDOW(window), _("Add Address to Book") );
-	gtk_window_set_position( GTK_WINDOW(window), GTK_WIN_POS_MOUSE );
-	gtk_window_set_modal( GTK_WINDOW(window), TRUE );
-	gtk_signal_connect( GTK_OBJECT(window), "delete_event",
-			    GTK_SIGNAL_FUNC(addressadd_delete_event), NULL );
-	gtk_signal_connect( GTK_OBJECT(window), "key_press_event",
-			    GTK_SIGNAL_FUNC(addressadd_key_pressed), NULL );
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_widget_set_size_request(window, 300, 360);
+	gtk_container_set_border_width(GTK_CONTAINER(window), 0);
+	gtk_window_set_title(GTK_WINDOW(window), _("Add Address to Book"));
+	gtk_window_set_position(GTK_WINDOW(window),
+				GTK_WIN_POS_CENTER_ON_PARENT);
+	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
+	gtk_widget_realize(window);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(addressadd_delete_event), NULL);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(addressadd_key_pressed), NULL);
 
-	vbox = gtk_vbox_new(FALSE, 8);
+	vbox = gtk_vbox_new(FALSE, 4);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 	gtk_container_set_border_width( GTK_CONTAINER(vbox), 0 );
 
 	table = gtk_table_new(3, 2, FALSE);
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
 	gtk_container_set_border_width( GTK_CONTAINER(table), 8 );
-	gtk_table_set_row_spacings(GTK_TABLE(table), 8);
-	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
+	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 4);
 
 	/* First row */
 	top = 0;
@@ -211,14 +203,14 @@ static void addressadd_create( void ) {
 	gtk_misc_set_alignment(GTK_MISC(label_rems), 0, 0.5);
 
 	/* Address book/folder tree */
-	vlbox = gtk_vbox_new(FALSE, 8);
+	vlbox = gtk_vbox_new(FALSE, 4);
 	gtk_box_pack_start(GTK_BOX(vbox), vlbox, TRUE, TRUE, 0);
-	gtk_container_set_border_width( GTK_CONTAINER(vlbox), 8 );
+	gtk_container_set_border_width( GTK_CONTAINER(vlbox), 4 );
 
 	tree_win = gtk_scrolled_window_new( NULL, NULL );
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(tree_win),
 				        GTK_POLICY_AUTOMATIC,
-				        GTK_POLICY_ALWAYS );
+				        GTK_POLICY_AUTOMATIC );
 	gtk_box_pack_start( GTK_BOX(vlbox), tree_win, TRUE, TRUE, 0 );
 
 	tree_folder = gtk_ctree_new( 1, 0 );
@@ -230,33 +222,27 @@ static void addressadd_create( void ) {
 	gtk_ctree_set_expander_style( GTK_CTREE(tree_folder), GTK_CTREE_EXPANDER_SQUARE );
 	gtk_ctree_set_indent( GTK_CTREE(tree_folder), CTREE_INDENT );
 	gtk_clist_set_auto_sort( GTK_CLIST(tree_folder), TRUE );
-
-	/* Status line */
-	hsbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, BORDER_WIDTH);
-	statusbar = gtk_statusbar_new();
-	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, BORDER_WIDTH);
+	gtkut_clist_set_redraw( GTK_CLIST(tree_folder) );
 
 	/* Button panel */
-	gtkut_button_set_create(&hbbox, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&hbbox, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
-	gtk_container_set_border_width( GTK_CONTAINER(hbbox), 0 );
+	gtk_container_set_border_width( GTK_CONTAINER(hbbox), 4 );
 	gtk_widget_grab_default(ok_btn);
 
 	hsep = gtk_hseparator_new();
 	gtk_box_pack_end(GTK_BOX(vbox), hsep, FALSE, FALSE, 0);
 
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(addressadd_ok), NULL);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(addressadd_cancel), NULL);
-	gtk_signal_connect(GTK_OBJECT(tree_folder), "select_row",
-			   GTK_SIGNAL_FUNC(addressadd_folder_select), NULL);
-	gtk_signal_connect(GTK_OBJECT(tree_folder), "button_press_event",
-			   GTK_SIGNAL_FUNC(addressadd_tree_button), NULL);
-
-	gtk_widget_show_all(vbox);
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(addressadd_ok), NULL);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(addressadd_cancel), NULL);
+	g_signal_connect(G_OBJECT(tree_folder), "select_row",
+			 G_CALLBACK(addressadd_folder_select), NULL);
+	g_signal_connect(G_OBJECT(tree_folder), "button_press_event",
+			 G_CALLBACK(addressadd_tree_button), NULL);
 
 	addressadd_dlg.window        = window;
 	addressadd_dlg.label_name    = label_name;
@@ -265,13 +251,12 @@ static void addressadd_create( void ) {
 	addressadd_dlg.tree_folder   = tree_folder;
 	addressadd_dlg.ok_btn        = ok_btn;
 	addressadd_dlg.cancel_btn    = cancel_btn;
-	addressadd_dlg.statusbar     = statusbar;
-	addressadd_dlg.status_cid    = gtk_statusbar_get_context_id( GTK_STATUSBAR(statusbar), "Address Add" );
 
-	gtk_widget_show_all( window );
+	gtk_widget_show_all(vbox);
 
-	PIXMAP_CREATE( window, bookXpm, bookXpmMask, book_xpm );
-	PIXMAP_CREATE( window, folderXpm, folderXpmMask, dir_open_xpm );
+	stock_pixmap_gdk( window, STOCK_PIXMAP_BOOK, &bookXpm, &bookXpmMask );
+	stock_pixmap_gdk( window, STOCK_PIXMAP_DIR_OPEN,
+			  &folderXpm, &folderXpmMask );
 }
 
 static void addressadd_load_folder( GtkCTreeNode *parentNode, ItemFolder *parentFolder,
@@ -358,11 +343,10 @@ gboolean addressadd_selection( AddressIndex *addrIndex, const gchar *name, const
 	addressadd_cancelled = FALSE;
 	if( ! addressadd_dlg.window ) addressadd_create();
 	gtk_widget_grab_focus(addressadd_dlg.ok_btn);
-	gtk_widget_show(addressadd_dlg.window);
 	manage_window_set_transient(GTK_WINDOW(addressadd_dlg.window));
+	gtk_widget_show(addressadd_dlg.window);
 
 	addressadd_dlg.fiSelected = NULL;
-	addressadd_status_show( "" );
 	addressadd_load_data( addrIndex );
 	gtk_clist_select_row( GTK_CLIST( addressadd_dlg.tree_folder ), 0, 0 );
 	gtk_widget_show(addressadd_dlg.window);
@@ -384,7 +368,12 @@ gboolean addressadd_selection( AddressIndex *addrIndex, const gchar *name, const
 		if( addressadd_dlg.fiSelected ) {
 			FolderInfo *fi = addressadd_dlg.fiSelected;
 			person = addrbook_add_contact( fi->book, fi->folder, name, address, remarks );
-			if( person ) retVal = TRUE;
+			if( person ) {
+				if( addressbook_edit_person( fi->book, NULL, person, FALSE ) == NULL )
+					addrbook_remove_person( fi->book, person );
+				else
+					retVal = TRUE;
+			}
 		}
 	}
 
