@@ -30,6 +30,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtksignal.h>
@@ -37,8 +38,12 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkbutton.h>
+#include <gtk/gtkstatusbar.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtknotebook.h>
+#include <gtk/gtkoptionmenu.h>
 
-#include "intl.h"
 #include "addressbook.h"
 #include "prefs_common.h"
 #include "addressitem.h"
@@ -57,6 +62,7 @@
 static struct _LDAPEdit {
 	GtkWidget *window;
 	GtkWidget *notebook;
+	GtkWidget *hbbox;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
 	GtkWidget *statusbar;
@@ -100,11 +106,12 @@ static gint edit_ldap_delete_event( GtkWidget *widget, GdkEventAny *event, gbool
 	return TRUE;
 }
 
-static void edit_ldap_key_pressed( GtkWidget *widget, GdkEventKey *event, gboolean *cancelled ) {
+static gboolean edit_ldap_key_pressed( GtkWidget *widget, GdkEventKey *event, gboolean *cancelled ) {
 	if (event && event->keyval == GDK_Escape) {
 		*cancelled = TRUE;
 		gtk_main_quit();
 	}
+	return FALSE;
 }
 
 static void edit_ldap_switch_page( GtkWidget *widget ) {
@@ -209,18 +216,18 @@ static void addressbook_edit_ldap_dialog_create( gboolean *cancelled ) {
 	GtkWidget *hsbox;
 	GtkWidget *statusbar;
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_widget_set_usize(window, 450, -1);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_widget_set_size_request(window, 450, -1);
 	gtk_container_set_border_width(GTK_CONTAINER(window), 0);
 	gtk_window_set_title(GTK_WINDOW(window), _("Edit LDAP Server"));
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);	
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(edit_ldap_delete_event),
-			   cancelled);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(edit_ldap_key_pressed),
-			   cancelled);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(edit_ldap_delete_event),
+			 cancelled);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(edit_ldap_key_pressed),
+			 cancelled);
 
 	vbox = gtk_vbox_new( FALSE, 6 );
 	/* gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER_WIDTH); */
@@ -235,27 +242,29 @@ static void addressbook_edit_ldap_dialog_create( gboolean *cancelled ) {
 
 	/* Status line */
 	hsbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, BORDER_WIDTH);
+	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, 0);
 	statusbar = gtk_statusbar_new();
-	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, BORDER_WIDTH);
+	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, 0);
 
 	/* Button panel */
-	gtkut_button_set_create(&hbbox, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&hbbox, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
 
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_ldap_ok), cancelled);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_ldap_cancel), cancelled);
-	gtk_signal_connect(GTK_OBJECT(notebook), "switch_page",
-			   GTK_SIGNAL_FUNC(edit_ldap_switch_page), NULL );
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(edit_ldap_ok), cancelled);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(edit_ldap_cancel), cancelled);
+	g_signal_connect(G_OBJECT(notebook), "switch_page",
+			 G_CALLBACK(edit_ldap_switch_page), NULL );
 
 	gtk_widget_show_all(vbox);
 
 	ldapedit.window     = window;
 	ldapedit.notebook   = notebook;
+	ldapedit.hbbox      = hbbox;
 	ldapedit.ok_btn     = ok_btn;
 	ldapedit.cancel_btn = cancel_btn;
 	ldapedit.statusbar  = statusbar;
@@ -321,7 +330,7 @@ void addressbook_edit_ldap_page_basic( gint pageNum, gchar *pageLbl ) {
 	spinbtn_port_adj = gtk_adjustment_new (389, 1, 65535, 100, 1000, 1000);
 	spinbtn_port = gtk_spin_button_new(GTK_ADJUSTMENT (spinbtn_port_adj), 1, 0);
 	gtk_box_pack_start (GTK_BOX (hbox_spin), spinbtn_port, FALSE, FALSE, 0);
-	gtk_widget_set_usize (spinbtn_port, 64, -1);
+	gtk_widget_set_size_request (spinbtn_port, 64, -1);
 	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_port), TRUE);
 	gtk_table_attach(GTK_TABLE(table), hbox_spin, 1, 2, top, (top + 1), GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
 
@@ -340,10 +349,10 @@ void addressbook_edit_ldap_page_basic( gint pageNum, gchar *pageLbl ) {
 	lookdn_btn = gtk_button_new_with_label( _(" ... "));
 	gtk_table_attach(GTK_TABLE(table), lookdn_btn, 2, 3, top, (top + 1), GTK_FILL, 0, 3, 0);
 
-	gtk_signal_connect(GTK_OBJECT(check_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_ldap_server_check), NULL);
-	gtk_signal_connect(GTK_OBJECT(lookdn_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_ldap_basedn_select), NULL);
+	g_signal_connect(G_OBJECT(check_btn), "clicked",
+			 G_CALLBACK(edit_ldap_server_check), NULL);
+	g_signal_connect(G_OBJECT(lookdn_btn), "clicked",
+			 G_CALLBACK(edit_ldap_basedn_select), NULL);
 
 	gtk_widget_show_all(vbox);
 
@@ -414,6 +423,7 @@ void addressbook_edit_ldap_page_extended( gint pageNum, gchar *pageLbl ) {
 
 	entry_bindPW = gtk_entry_new();
 	gtk_table_attach(GTK_TABLE(table), entry_bindPW, 1, 2, top, (top + 1), GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
+	gtk_entry_set_visibility(GTK_ENTRY(entry_bindPW), FALSE);
 
 	/* Next row */
 	++top;
@@ -425,7 +435,7 @@ void addressbook_edit_ldap_page_extended( gint pageNum, gchar *pageLbl ) {
 	spinbtn_timeout_adj = gtk_adjustment_new (0, 0, 300, 1, 10, 10);
 	spinbtn_timeout = gtk_spin_button_new(GTK_ADJUSTMENT (spinbtn_timeout_adj), 1, 0);
 	gtk_box_pack_start (GTK_BOX (hbox_spin), spinbtn_timeout, FALSE, FALSE, 0);
-	gtk_widget_set_usize (spinbtn_timeout, 64, -1);
+	gtk_widget_set_size_request (spinbtn_timeout, 64, -1);
 	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_timeout), TRUE);
 	gtk_table_attach(GTK_TABLE(table), hbox_spin, 1, 2, top, (top + 1), GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
 
@@ -439,12 +449,12 @@ void addressbook_edit_ldap_page_extended( gint pageNum, gchar *pageLbl ) {
 	spinbtn_maxentry_adj = gtk_adjustment_new (0, 0, 500, 1, 10, 10);
 	spinbtn_maxentry = gtk_spin_button_new(GTK_ADJUSTMENT (spinbtn_maxentry_adj), 1, 0);
 	gtk_box_pack_start (GTK_BOX (hbox_spin), spinbtn_maxentry, FALSE, FALSE, 0);
-	gtk_widget_set_usize (spinbtn_maxentry, 64, -1);
+	gtk_widget_set_size_request (spinbtn_maxentry, 64, -1);
 	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_maxentry), TRUE);
 	gtk_table_attach(GTK_TABLE(table), hbox_spin, 1, 2, top, (top + 1), GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
 
-	gtk_signal_connect(GTK_OBJECT(reset_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_ldap_search_reset), NULL);
+	g_signal_connect(G_OBJECT(reset_btn), "clicked",
+			 G_CALLBACK(edit_ldap_search_reset), NULL);
 
 	gtk_widget_show_all(vbox);
 
@@ -475,7 +485,7 @@ void edit_ldap_set_optmenu( GtkOptionMenu *optmenu, const gint value ) {
 	menu = gtk_option_menu_get_menu(optmenu);
 	for( cur = GTK_MENU_SHELL(menu)->children; cur != NULL; cur = cur->next ) {
 		menuitem = GTK_WIDGET(cur->data);
-		menuVal = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		menuVal = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menuitem), "user_data"));
 		if( menuVal == value ) {
 			gtk_option_menu_set_history(optmenu, n);
 			return;
@@ -493,7 +503,7 @@ gint edit_ldap_get_optmenu( GtkOptionMenu *optmenu ) {
 
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(optmenu));
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
-	return GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+	return GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menuitem), "user_data"));
 }
 
 AdapterDSource *addressbook_edit_ldap( AddressIndex *addrIndex, AdapterDSource *ads ) {
@@ -506,7 +516,9 @@ AdapterDSource *addressbook_edit_ldap( AddressIndex *addrIndex, AdapterDSource *
 
 	if (!ldapedit.window)
 		addressbook_edit_ldap_create(&cancelled);
-	gtk_notebook_set_page( GTK_NOTEBOOK(ldapedit.notebook), 0 );
+	gtkut_box_set_reverse_order(GTK_BOX(ldapedit.hbbox),
+				    !prefs_common.comply_gnome_hig);
+	gtk_notebook_set_current_page( GTK_NOTEBOOK(ldapedit.notebook), 0 );
 	gtk_widget_grab_focus(ldapedit.ok_btn);
 	gtk_widget_grab_focus(ldapedit.entry_name);
 	gtk_widget_show(ldapedit.window);
