@@ -53,6 +53,8 @@ struct _FolderView
 	GtkTreeRowReference *opened;
 	GtkTreeRowReference *prev_selected;
 
+	gboolean display_folder_unread;
+
 	gboolean open_folder;
 
 	guint expand_timeout;
@@ -69,6 +71,7 @@ struct _FolderView
 
 FolderView *folderview_create		(void);
 void folderview_init			(FolderView	*folderview);
+void folderview_reflect_prefs		(FolderView	*folderview);
 
 FolderView *folderview_get		(void);
 
@@ -91,8 +94,9 @@ gboolean folderview_append_item		(FolderView	*folderview,
 					 FolderItem	*item,
 					 gboolean	 expand_parent);
 
-void folderview_check_new		(Folder		*folder);
-void folderview_check_new_all		(void);
+gint folderview_check_new		(Folder		*folder);
+gint folderview_check_new_item		(FolderItem	*item);
+gint folderview_check_new_all		(void);
 
 void folderview_update_item		(FolderItem	*item,
 					 gboolean	 update_summary);
