@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -97,6 +97,7 @@ static struct Receive {
 	GtkWidget *imap_frame;
 	GtkWidget *imap_auth_type_optmenu;
 	GtkWidget *imap_check_inbox_chkbtn;
+	GtkWidget *imap_filter_inbox_chkbtn;
 
 	GtkWidget *nntp_frame;
 	GtkWidget *maxarticle_spinbtn;
@@ -267,6 +268,8 @@ static PrefsUIData ui_data[] = {
 	 prefs_set_data_from_entry, prefs_set_entry},
 	{"imap_check_inbox_only", &receive.imap_check_inbox_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
+	{"imap_filter_inbox_on_receive", &receive.imap_filter_inbox_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"imap_auth_method", &receive.imap_auth_type_optmenu,
 	 prefs_account_imap_auth_type_set_data_from_optmenu,
 	 prefs_account_imap_auth_type_set_optmenu},
@@ -839,6 +842,7 @@ static void prefs_account_receive_create(void)
 	GtkWidget *optmenu_menu;
 	GtkWidget *menuitem;
 	GtkWidget *imap_check_inbox_chkbtn;
+	GtkWidget *imap_filter_inbox_chkbtn;
 
 	GtkWidget *nntp_frame;
 	GtkWidget *maxarticle_label;
@@ -991,6 +995,8 @@ static void prefs_account_receive_create(void)
 
 	PACK_CHECK_BUTTON (vbox2, imap_check_inbox_chkbtn,
 			   _("Only check INBOX on receiving"));
+	PACK_CHECK_BUTTON (vbox2, imap_filter_inbox_chkbtn,
+			   _("Filter new messages in INBOX on receiving"));
 
 	PACK_FRAME (vbox1, nntp_frame, _("News"));
 
@@ -1039,9 +1045,10 @@ static void prefs_account_receive_create(void)
 	receive.inbox_entry           = inbox_entry;
 	receive.inbox_btn             = inbox_btn;
 
-	receive.imap_frame              = imap_frame;
-	receive.imap_auth_type_optmenu  = optmenu;
-	receive.imap_check_inbox_chkbtn = imap_check_inbox_chkbtn;
+	receive.imap_frame               = imap_frame;
+	receive.imap_auth_type_optmenu   = optmenu;
+	receive.imap_check_inbox_chkbtn  = imap_check_inbox_chkbtn;
+	receive.imap_filter_inbox_chkbtn = imap_filter_inbox_chkbtn;
 
 	receive.nntp_frame             = nntp_frame;
 	receive.maxarticle_spinbtn     = maxarticle_spinbtn;
@@ -1393,9 +1400,9 @@ static void prefs_account_privacy_create(void)
 	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);
 
 	PACK_CHECK_BUTTON (vbox2, default_sign_chkbtn,
-			   _("Sign message by default"));
+			   _("PGP sign message by default"));
 	PACK_CHECK_BUTTON (vbox2, default_encrypt_chkbtn,
-			   _("Encrypt message by default"));
+			   _("PGP encrypt message by default"));
 	PACK_CHECK_BUTTON (vbox2, encrypt_reply_chkbtn,
 			   _("Encrypt when replying to encrypted message"));
 	PACK_CHECK_BUTTON (vbox2, ascii_armored_chkbtn,
@@ -1498,6 +1505,58 @@ static void prefs_account_privacy_create(void)
 	CREATE_RADIO_BUTTON(box, btn3, btn1, btn3_label, btn3_data);	\
 }
 
+static void pop_ssltunnel_toggled(GtkToggleButton *button, gpointer data)
+{
+	if (gtk_toggle_button_get_active
+		(GTK_TOGGLE_BUTTON(advanced.popport_chkbtn)))
+		return;
+
+	if (gtk_toggle_button_get_active(button)) {
+		gtk_entry_set_text(GTK_ENTRY(advanced.popport_entry), "995");
+	} else {
+		gtk_entry_set_text(GTK_ENTRY(advanced.popport_entry), "110");
+	}
+}
+
+static void imap_ssltunnel_toggled(GtkToggleButton *button, gpointer data)
+{
+	if (gtk_toggle_button_get_active
+		(GTK_TOGGLE_BUTTON(advanced.imapport_chkbtn)))
+		return;
+
+	if (gtk_toggle_button_get_active(button)) {
+		gtk_entry_set_text(GTK_ENTRY(advanced.imapport_entry), "993");
+	} else {
+		gtk_entry_set_text(GTK_ENTRY(advanced.imapport_entry), "143");
+	}
+}
+
+static void nntp_ssltunnel_toggled(GtkToggleButton *button, gpointer data)
+{
+	if (gtk_toggle_button_get_active
+		(GTK_TOGGLE_BUTTON(advanced.nntpport_chkbtn)))
+		return;
+
+	if (gtk_toggle_button_get_active(button)) {
+		gtk_entry_set_text(GTK_ENTRY(advanced.nntpport_entry), "563");
+	} else {
+		gtk_entry_set_text(GTK_ENTRY(advanced.nntpport_entry), "119");
+	}
+}
+
+static void smtp_ssltunnel_toggled(GtkToggleButton *button, gpointer data)
+{
+	if (gtk_toggle_button_get_active
+		(GTK_TOGGLE_BUTTON(advanced.smtpport_chkbtn)))
+		return;
+
+	if (gtk_toggle_button_get_active(button)) {
+		gtk_entry_set_text(GTK_ENTRY(advanced.smtpport_entry), "465");
+	} else {
+		gtk_entry_set_text(GTK_ENTRY(advanced.smtpport_entry), "25");
+	}
+}
+
 static void prefs_account_ssl_create(void)
 {
 	GtkWidget *vbox1;
@@ -1551,6 +1610,9 @@ static void prefs_account_ssl_create(void)
 			     _("Use STARTTLS command to start SSL session"),
 			     SSL_STARTTLS);
 
+	g_signal_connect(G_OBJECT(pop_ssltunnel_radiobtn), "toggled",
+			 G_CALLBACK(pop_ssltunnel_toggled), NULL);
+
 	PACK_FRAME (vbox1, imap_frame, _("IMAP4"));
 	vbox3 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox3);
@@ -1568,6 +1630,9 @@ static void prefs_account_ssl_create(void)
 			     _("Use STARTTLS command to start SSL session"),
 			     SSL_STARTTLS);
 
+	g_signal_connect(G_OBJECT(imap_ssltunnel_radiobtn), "toggled",
+			 G_CALLBACK(imap_ssltunnel_toggled), NULL);
+
 	PACK_FRAME (vbox1, nntp_frame, _("NNTP"));
 	vbox4 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox4);
@@ -1585,6 +1650,9 @@ static void prefs_account_ssl_create(void)
 	CREATE_RADIO_BUTTON(vbox4, nntp_ssltunnel_radiobtn, nntp_nossl_radiobtn,
 			    _("Use SSL for NNTP connection"), SSL_TUNNEL);
 
+	g_signal_connect(G_OBJECT(nntp_ssltunnel_radiobtn), "toggled",
+			 G_CALLBACK(nntp_ssltunnel_toggled), NULL);
+
 	PACK_FRAME (vbox1, send_frame, _("Send (SMTP)"));
 	vbox5 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox5);
@@ -1602,6 +1670,9 @@ static void prefs_account_ssl_create(void)
 			     _("Use STARTTLS command to start SSL session"),
 			     SSL_STARTTLS);
 
+	g_signal_connect(G_OBJECT(smtp_ssltunnel_radiobtn), "toggled",
+			 G_CALLBACK(smtp_ssltunnel_toggled), NULL);
+
 	vbox6 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox6);
 	gtk_box_pack_start (GTK_BOX (vbox1), vbox6, FALSE, FALSE, 0);
