@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -149,9 +149,9 @@ static gboolean row_selected		(GtkTreeSelection	*selection,
 					 gboolean		 cur_selected,
 					 gpointer		 data);
 
-static void query_search_clear	(GtkButton	*button,
+static void query_search_clear		(GtkButton	*button,
 					 gpointer	 data);
-static void query_select_folder	(GtkButton	*button,
+static void query_select_folder		(GtkButton	*button,
 					 gpointer	 data);
 static void query_search_clicked	(GtkButton	*button,
 					 gpointer	 data);
@@ -574,6 +574,9 @@ static void query_search_folder(FolderItem *item)
 		return;
 	}
 
+	if (item->opened)
+		summary_write_cache(main_window_get()->summaryview);
+
 	mlist = folder_item_get_msg_list(item, TRUE);
 	total = g_slist_length(mlist);
 
@@ -970,7 +973,7 @@ static FolderItem *query_search_create_vfolder(FolderItem *parent,
 	}
 
 	if (parent->path)
-		path = g_strconcat(parent->path, G_DIR_SEPARATOR_S, name, NULL);
+		path = g_strconcat(parent->path, "/", name, NULL);
 	else
 		path = g_strdup(name);
 
