@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,6 +54,8 @@
 #include "socket.h"
 #include "utils.h"
 #include "inc.h"
+#include "mainwindow.h"
+#include "summaryview.h"
 
 #define SMTP_PORT	25
 #if USE_SSL
@@ -126,7 +128,9 @@ enum
 	Q_SENDER     = 0,
 	Q_SMTPSERVER = 1,
 	Q_RECIPIENTS = 2,
-	Q_ACCOUNT_ID = 3
+	Q_ACCOUNT_ID = 3,
+	Q_REPLY_TARGET = 4,
+	Q_FORWARD_TARGETS = 5
 };
 
 QueueInfo *send_get_queue_info(const gchar *file)
@@ -135,6 +139,8 @@ QueueInfo *send_get_queue_info(const gchar *file)
 				       {"SSV:", NULL, FALSE},
 				       {"R:",   NULL, FALSE},
 				       {"AID:", NULL, FALSE},
+				       {"REP:", NULL, FALSE},
+				       {"FWD:", NULL, FALSE},
 				       {NULL,   NULL, FALSE}};
 	FILE *fp;
 	gchar buf[BUFFSIZE];
@@ -172,6 +178,12 @@ QueueInfo *send_get_queue_info(const gchar *file)
 		case Q_ACCOUNT_ID:
 			qinfo->ac = account_find_from_id(atoi(p));
 			break;
+		case Q_REPLY_TARGET:
+			qinfo->reply_target = g_strdup(p);
+			break;
+		case Q_FORWARD_TARGETS:
+			qinfo->forward_targets = g_strdup(p);
+			break;
 		default:
 			break;
 		}
@@ -238,6 +250,8 @@ void send_queue_info_free(QueueInfo *qinfo)
 	g_slist_free(qinfo->to_list);
 	g_free(qinfo->from);
 	g_free(qinfo->server);
+	g_free(qinfo->reply_target);
+	g_free(qinfo->forward_targets);
 	if (qinfo->fp)
 		fclose(qinfo->fp);
 	g_free(qinfo);
@@ -340,6 +354,12 @@ gint send_message_queue_all(FolderItem *queue, gboolean save_msgs,
 			continue;
 		}
 
+		if (qinfo->reply_target)
+			send_message_set_reply_flag(qinfo->reply_target,
+						    msginfo->inreplyto);
+		else if (qinfo->forward_targets)
+			send_message_set_forward_flags(qinfo->forward_targets);
+
 		g_snprintf(tmp, sizeof(tmp), "%s%ctmpmsg.out.%08x",
 			   get_rc_dir(), G_DIR_SEPARATOR, g_random_int());
 
@@ -388,6 +408,141 @@ gint send_message_queue_all(FolderItem *queue, gboolean save_msgs,
 	return ret;
 }
 
+gint send_message_set_reply_flag(const gchar *target, const gchar *msgid)
+{
+	FolderItem *item;
+	gint num;
+	MsgInfo *msginfo;
+	SummaryView *summaryview;
+
+	g_return_val_if_fail(target != NULL, -1);
+
+	debug_print("send_message_set_reply_flag(): "
+		    "setting reply flag to: %s\n", target);
+
+	summaryview = main_window_get()->summaryview;
+
+	item = folder_find_item_and_num_from_id(target, &num);
+	if (!item || num <= 0)
+		return -1;
+
+	if (summaryview->folder_item == item) {
+		msginfo = summary_get_msginfo_by_msgnum(summaryview, num);
+		if (msginfo) {
+			if (msgid && strcmp2(msgid, msginfo->msgid) != 0) {
+				debug_print("send_message_set_reply_flag(): "
+					    "message-id mismatch\n");
+				return -1;
+			}
+			MSG_UNSET_PERM_FLAGS(msginfo->flags, MSG_FORWARDED);
+			MSG_SET_PERM_FLAGS(msginfo->flags, MSG_REPLIED);
+			MSG_SET_TMP_FLAGS(msginfo->flags, MSG_FLAG_CHANGED);
+			if (MSG_IS_IMAP(msginfo->flags))
+				imap_msg_set_perm_flags(msginfo, MSG_REPLIED);
+			if (msginfo->folder)
+				msginfo->folder->mark_dirty = TRUE;
+			summary_update_by_msgnum(summaryview, msginfo->msgnum);
+                }
+	} else {
+		msginfo = procmsg_get_msginfo(item, num);
+		if (msginfo) {
+			if (msgid && strcmp2(msgid, msginfo->msgid) != 0) {
+				debug_print("send_message_set_reply_flag(): "
+					    "message-id mismatch\n");
+				procmsg_msginfo_free(msginfo);
+				return -1;
+			}
+			MSG_UNSET_PERM_FLAGS(msginfo->flags, MSG_FORWARDED);
+			MSG_SET_PERM_FLAGS(msginfo->flags, MSG_REPLIED);
+			MSG_SET_TMP_FLAGS(msginfo->flags, MSG_FLAG_CHANGED);
+			if (MSG_IS_IMAP(msginfo->flags))
+				imap_msg_set_perm_flags(msginfo, MSG_REPLIED);
+			if (msginfo->folder)
+				msginfo->folder->mark_dirty = TRUE;
+			procmsg_add_flags(msginfo->folder, msginfo->msgnum,
+					  msginfo->flags);
+			procmsg_msginfo_free(msginfo);
+                }
+	}
+
+	return 0;
+}
+
+gint send_message_set_forward_flags(const gchar *targets)
+{
+	FolderItem *item;
+	gint num;
+	MsgInfo *msginfo;
+	SummaryView *summaryview;
+	gchar **paths;
+	gint i;
+	GSList *mlist = NULL;
+
+	g_return_val_if_fail(targets != NULL, -1);
+
+	debug_print("send_message_set_forward_flags(): "
+		    "setting forward flags to: %s\n", targets);
+
+	summaryview = main_window_get()->summaryview;
+
+	paths = g_strsplit(targets, "\n", 0);
+
+	for (i = 0; paths[i] != NULL; i++) {
+		g_strstrip(paths[i]);
+
+		item = folder_find_item_and_num_from_id(paths[i], &num);
+		if (!item || num <= 0)
+			continue;
+
+		if (summaryview->folder_item == item) {
+			msginfo = summary_get_msginfo_by_msgnum(summaryview,
+								num);
+			if (msginfo) {
+				MSG_UNSET_PERM_FLAGS(msginfo->flags,
+						     MSG_REPLIED);
+				MSG_SET_PERM_FLAGS(msginfo->flags,
+						   MSG_FORWARDED);
+				MSG_SET_TMP_FLAGS(msginfo->flags,
+						  MSG_FLAG_CHANGED);
+				if (msginfo->folder)
+					msginfo->folder->mark_dirty = TRUE;
+				summary_update_by_msgnum
+					(summaryview, msginfo->msgnum);
+				msginfo = procmsg_msginfo_copy(msginfo);
+			}
+		} else {
+			msginfo = procmsg_get_msginfo(item, num);
+			if (msginfo) {
+				MSG_UNSET_PERM_FLAGS(msginfo->flags,
+						     MSG_REPLIED);
+				MSG_SET_PERM_FLAGS(msginfo->flags,
+						   MSG_FORWARDED);
+				MSG_SET_TMP_FLAGS(msginfo->flags,
+						  MSG_FLAG_CHANGED);
+				if (msginfo->folder)
+					msginfo->folder->mark_dirty = TRUE;
+				procmsg_add_flags(msginfo->folder,
+						  msginfo->msgnum,
+						  msginfo->flags);
+			}
+		}
+
+		if (msginfo)
+			mlist = g_slist_append(mlist, msginfo);
+	}
+
+	if (mlist) {
+		msginfo = (MsgInfo *)mlist->data;
+		if (MSG_IS_IMAP(msginfo->flags))
+			imap_msg_list_unset_perm_flags(mlist, MSG_REPLIED);
+	}
+
+	procmsg_msg_list_free(mlist);
+	g_strfreev(paths);
+
+	return 0;
+}
+
 static gint send_message_local(const gchar *command, FILE *fp)
 {
 	gchar **argv;
@@ -600,6 +755,11 @@ static gint send_message_smtp(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp)
 		/* consider EOF right after QUIT successful */
 		log_warning("%s\n", _("Connection closed by the remote host."));
 		ret = 0;
+	} else if (session->state == SESSION_ERROR &&
+		   SMTP_SESSION(session)->state == SMTP_QUIT) {
+		/* ignore errors right after QUIT */
+		log_warning("%s\n", _("Error occurred after QUIT command (ignored)"));
+		ret = 0;
 	} else if (session->state == SESSION_ERROR ||
 		   session->state == SESSION_EOF ||
 		   session->state == SESSION_TIMEOUT ||
