@@ -1,7 +1,7 @@
 /*
  * Sylpheed templates subsystem 
  * Copyright (C) 2001 Alexander Barinov
- * Copyright (C) 2001-2004 Hiroyuki Yamamoto
+ * Copyright (C) 2001-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,12 +21,11 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
-#include <dirent.h>
 #include <sys/stat.h>
 #include <ctype.h>
 
-#include "intl.h"
 #include "main.h"
 #include "template.h"
 #include "utils.h"
@@ -40,7 +39,7 @@ static Template *template_load(gchar *filename)
 	gchar buf[BUFFSIZE];
 	gint bytes_read;
 
-	if ((fp = fopen(filename, "rb")) == NULL) {
+	if ((fp = g_fopen(filename, "rb")) == NULL) {
 		FILE_OP_ERROR(filename, "fopen");
 		return NULL;
 	}
@@ -49,19 +48,25 @@ static Template *template_load(gchar *filename)
 	tmpl->name = NULL;
 	tmpl->to = NULL;
 	tmpl->cc = NULL;
+	tmpl->bcc = NULL;
+	tmpl->replyto = NULL;
 	tmpl->subject = NULL;
 	tmpl->value = NULL;
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
 		if (buf[0] == '\n')
 			break;
-		else if (!g_strncasecmp(buf, "Name:", 5))
+		else if (!g_ascii_strncasecmp(buf, "Name:", 5))
 			tmpl->name = g_strdup(g_strstrip(buf + 5));
-		else if (!g_strncasecmp(buf, "To:", 3))
+		else if (!g_ascii_strncasecmp(buf, "To:", 3))
 			tmpl->to = g_strdup(g_strstrip(buf + 3));
-		else if (!g_strncasecmp(buf, "Cc:", 3))
+		else if (!g_ascii_strncasecmp(buf, "Cc:", 3))
 			tmpl->cc = g_strdup(g_strstrip(buf + 3));
-		else if (!g_strncasecmp(buf, "Subject:", 8))
+		else if (!g_ascii_strncasecmp(buf, "Bcc:", 3))
+			tmpl->bcc = g_strdup(g_strstrip(buf + 4));
+		else if (!g_ascii_strncasecmp(buf, "Reply-To:", 9))
+			tmpl->replyto = g_strdup(g_strstrip(buf + 9));
+		else if (!g_ascii_strncasecmp(buf, "Subject:", 8))
 			tmpl->subject = g_strdup(g_strstrip(buf + 8));
 	}
 
@@ -108,10 +113,10 @@ void template_clear_config(GSList *tmpl_list)
 
 GSList *template_read_config(void)
 {
-	gchar *path;
+	const gchar *path;
 	gchar *filename;
-	DIR *dp;
-	struct dirent *de;
+	GDir *dir;
+	const gchar *dir_name;
 	struct stat s;
 	Template *tmpl;
 	GSList *tmpl_list = NULL;
@@ -125,17 +130,16 @@ GSList *template_read_config(void)
 			return NULL;
 	}
 
-	if ((dp = opendir(path)) == NULL) {
-		FILE_OP_ERROR(path, "opendir");
+	if ((dir = g_dir_open(path, 0, NULL)) == NULL) {
+		g_warning("failed to open directory: %s\n", path);
 		return NULL;
 	}
 
-	while ((de = readdir(dp)) != NULL) {
-		if (*de->d_name == '.') continue;
+	while ((dir_name = g_dir_read_name(dir)) != NULL) {
+		filename = g_strconcat(path, G_DIR_SEPARATOR_S,
+				       dir_name, NULL);
 
-		filename = g_strconcat(path, G_DIR_SEPARATOR_S, de->d_name, NULL);
-
-		if (stat(filename, &s) != 0 || !S_ISREG(s.st_mode) ) {
+		if (g_stat(filename, &s) != 0 || !S_ISREG(s.st_mode) ) {
 			debug_print("%s:%d %s is not an ordinary file\n",
 				    __FILE__, __LINE__, filename);
 			continue;
@@ -144,17 +148,18 @@ GSList *template_read_config(void)
 		tmpl = template_load(filename);
 		if (tmpl)
 			tmpl_list = g_slist_append(tmpl_list, tmpl);
+
 		g_free(filename);
 	}
 
-	closedir(dp);
+	g_dir_close(dir);
 
 	return tmpl_list;
 }
 
 void template_write_config(GSList *tmpl_list)
 {
-	gchar *path;
+	const gchar *path;
 	GSList *cur;
 	Template *tmpl;
 	FILE *fp;
@@ -184,7 +189,7 @@ void template_write_config(GSList *tmpl_list)
 		filename = g_strconcat(path, G_DIR_SEPARATOR_S,
 				       itos(tmpl_num), NULL);
 
-		if ((fp = fopen(filename, "wb")) == NULL) {
+		if ((fp = g_fopen(filename, "wb")) == NULL) {
 			FILE_OP_ERROR(filename, "fopen");
 			g_free(filename);
 			return;
@@ -195,10 +200,15 @@ void template_write_config(GSList *tmpl_list)
 			fprintf(fp, "To: %s\n", tmpl->to);
 		if (tmpl->cc && *tmpl->cc != '\0')
 			fprintf(fp, "Cc: %s\n", tmpl->cc);
+		if (tmpl->bcc && *tmpl->bcc != '\0')
+			fprintf(fp, "Bcc: %s\n", tmpl->bcc);
+		if (tmpl->replyto && *tmpl->replyto != '\0')
+			fprintf(fp, "Reply-To: %s\n", tmpl->replyto);
 		if (tmpl->subject && *tmpl->subject != '\0')
 			fprintf(fp, "Subject: %s\n", tmpl->subject);
 		fputs("\n", fp);
 		fwrite(tmpl->value, sizeof(gchar) * strlen(tmpl->value), 1, fp);
+
 		fclose(fp);
 		g_free(filename);
 	}
