@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -489,7 +489,19 @@ void textview_show_message(TextView *textview, MimeInfo *mimeinfo,
 		textview->body_pos = gtk_text_iter_get_offset(&iter);
 	}
 
+#if USE_GPGME
+	if (textview->messageview->msginfo->encinfo &&
+	    textview->messageview->msginfo->encinfo->decryption_failed) {
+		gtk_text_buffer_get_end_iter(buffer, &iter);
+		gtk_text_buffer_insert(buffer, &iter, "\n", 1);
+		gtk_text_buffer_insert_with_tags_by_name
+			(buffer, &iter, _("This message is encrypted, but its decryption failed.\n"),
+			 -1, "error", "mimepart", NULL);
+	}
+#endif
+
 	textview_add_parts(textview, mimeinfo, fp);
+
 #if USE_GPGME
 	if (textview->messageview->msginfo->encinfo &&
 	    textview->messageview->msginfo->encinfo->sigstatus)
@@ -990,11 +1002,13 @@ static gchar *make_email_string(const gchar *bp, const gchar *ep)
 {
 	/* returns a mailto: URI; mailto: is also used to detect the
 	 * uri type later on in the button_pressed signal handler */
-	gchar *tmp;
+	gchar *tmp, *enc;
 	gchar *result;
 
 	tmp = g_strndup(bp, ep - bp);
-	result = g_strconcat("mailto:", tmp, NULL);
+	enc = uriencode_for_mailto(tmp);
+	result = g_strconcat("mailto:", enc, NULL);
+	g_free(enc);
 	g_free(tmp);
 
 	return result;
@@ -1256,6 +1270,10 @@ void textview_clear(TextView *textview)
 	buffer = gtk_text_view_get_buffer(text);
 	gtk_text_buffer_set_text(buffer, "", -1);
 
+	/* workaround for the assertion failure in
+	   gtk_text_view_validate_onscreen() */
+	text->vadjustment->value = 0.0;
+
 	STATUSBAR_POP(textview);
 	textview_uri_list_remove_all(textview->uri_list);
 	textview->uri_list = NULL;
@@ -2040,26 +2058,29 @@ static void textview_popup_menu_activate_add_address_cb(GtkMenuItem *menuitem,
 							gpointer data)
 {
 	RemoteURI *uri = (RemoteURI *)data;
-	const gchar *addr;
+	gchar *addr;
 
 	g_return_if_fail(uri != NULL);
 
 	if (!uri->uri)
 		return;
 
-	if (!g_ascii_strncasecmp(uri->uri, "mailto:", 7))
-		addr = uri->uri + 7;
-	else
-		addr = uri->uri;
+	if (!g_ascii_strncasecmp(uri->uri, "mailto:", 7)) {
+		addr = g_malloc(strlen(uri->uri + 7) + 1);
+		decode_uri(addr, uri->uri + 7);
+	} else
+		addr = g_strdup(uri->uri);
 
 	addressbook_add_contact(addr, addr, NULL);
+
+	g_free(addr);
 }
 
 static void textview_popup_menu_activate_copy_cb(GtkMenuItem *menuitem,
 						 gpointer data)
 {
 	RemoteURI *uri = (RemoteURI *)data;
-	const gchar *uri_string;
+	gchar *uri_string;
 	GtkClipboard *clipboard;
 
 	g_return_if_fail(uri != NULL);
@@ -2067,15 +2088,18 @@ static void textview_popup_menu_activate_copy_cb(GtkMenuItem *menuitem,
 	if (!uri->uri)
 		return;
 
-	if (!g_ascii_strncasecmp(uri->uri, "mailto:", 7))
-		uri_string = uri->uri + 7;
-	else
-		uri_string = uri->uri;
+	if (!g_ascii_strncasecmp(uri->uri, "mailto:", 7)) {
+		uri_string = g_malloc(strlen(uri->uri + 7) + 1);
+		decode_uri(uri_string, uri->uri + 7);
+	} else
+		uri_string = g_strdup(uri->uri);
 
 	clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
 	gtk_clipboard_set_text(clipboard, uri_string, -1);
 	clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
 	gtk_clipboard_set_text(clipboard, uri_string, -1);
+
+	g_free(uri_string);
 }
 
 static void textview_popup_menu_activate_image_cb(GtkMenuItem *menuitem,
